local _, class = UnitClass("player");

local locale = {
	LARMOR="Armor:",
	LCRIT="Crit:",
	LDODGE="Dodge:",
	LPARRY="Parry:",
	LMISS="Miss:",
	LBLOCK="Block:",
	LHIT="Hit:",
	ARMOR="|cffffffffDamage Reduction: %.2f%%|r",
	ARMOR2="Armor-based physical damage reduction.";
	MISS="|cffffffffChance to be missed: %.2f%%|r",
	MISS2="|cffacacac(Affected by diminishing returns)|r",
	CRIT="|cffffffffCritical hit chance: %.2f%%|r",
	DODGE="|cffffffffDodge chance: %.2f%%|r",
	PARRY="|cffffffffParry chance: %.2f%%|r",
	BLOCK="|cffffffffBlock chance: %.2f%%|r",
	BLOCK2="Blocking stops %d damage.",
	HIT="|cffffffffHit chance: %.2f%%|r",
	HIT2="Hits deal normal damage.",
	ALT="Versus level %d opponents: %.2f%%",
	SLASH="Versus level %d opponents: %.2f%% crit, %.2f%% hit, %.2f%% miss, %.2f%% dodge, %.2f%% parry, %.2f%% block. %.2f%% damage mitigated by armor."
}
HT_HITTABLE = "Inc. Hit Table";

local function getArmorMitigation(delta)
	local level, _, armor = UnitLevel("player") + (delta or 0), UnitArmor("player");
	if level < 60 then
		return math.min(75, 100 * armor / (armor + 400 + 85 * level));
	end
	return math.min(75, 100 * armor / (armor + 400 + 85 * (level + 4.5 *  (level - 59))));
end
local function getDefenseSkillDelta()
	local level, base, bonus = UnitLevel("player")*5, UnitDefense("player");
	return base+bonus-level;
end
local function getCritChance(delta)
	local talentBonus = class == "DRUID" and select(5,GetTalentInfo(2, 18))*2 or 0;
	return math.max(0, 5 + delta*0.20 - GetCombatRatingBonus(CR_CRIT_TAKEN_MELEE) - getDefenseSkillDelta() * 0.04 - talentBonus);
end
local function getMissChance(delta) -- This one is wrong: the miss component of defense should diminish.
	return math.max(0, 5 - delta*0.20 + getDefenseSkillDelta()*0.04);
end

local function getIncomingHitTable(levelDelta)
	levelDelta = levelDelta or 0;
	local crit, miss, dodge, parry, block = 
		getCritChance(levelDelta), getMissChance(levelDelta), GetDodgeChance(), GetParryChance(), GetBlockChance();
	local hit = math.max(0, 100-miss-dodge-parry-block);
	return math.min(hit, crit), hit, miss, dodge, parry, block, getArmorMitigation(levelDelta);
end

local statLine = 1;
local function HT_SetStat(prefix, label, value, v2, v3, tip, tip2)
	if statLine > 6 or value == 0 then return end
	local lbl, val, parent = _G[prefix .. statLine .. "Label"], _G[prefix .. statLine .. "StatText"], _G[prefix .. statLine];
	local lvl = UnitLevel("player");
	if value == nil then
		parent.tooltip = nil; lbl:SetText(""); val:SetText("");
	else
		lbl:SetText(label); val:SetFormattedText("%.2f%%", value);
		parent.tooltip, parent.tooltip2, parent.tooltip3 = tip and tip:format(value) or nil, tip2 or nil;
		if v2 ~= value and v2 then
			parent.tooltip2 = (parent.tooltip2 and (parent.tooltip2 .. "\n") or "") .. locale.ALT:format(lvl+2, v2);
		end
		if v3 ~= value and v3 then
			parent.tooltip2 = (parent.tooltip2 and (parent.tooltip2 .. "\n") or "") .. locale.ALT:format(lvl, v3);
		end
	end
 	statLine = statLine + 1;
end
local function HT_Hook(prefix, index)
	if index ~= "HT_HITTABLE" then
		return;
	end
	local critP3, hitP3, missP3, dodgeP3, parryP3, blockP3, armor3 = getIncomingHitTable(3);
	local critP2, hitP2, missP2, dodgeP2, parryP2, blockP2, armor2 = getIncomingHitTable(2);
	local critP0, hitP0, missP0, dodgeP0, parryP0, blockP0, armor0 = getIncomingHitTable(0);
	
	statLine = 1;
	HT_SetStat(prefix, locale.LARMOR, armor3, armor2, armor0, locale.ARMOR, locale.ARMOR2);
	HT_SetStat(prefix, locale.LCRIT, critP3, critP2, critP0, locale.CRIT, locale.CRIT2);
	HT_SetStat(prefix, locale.LDODGE, dodgeP3, nil, nil, locale.DODGE, locale.DODGE2);
	HT_SetStat(prefix, locale.LPARRY, parryP3, nil, nil, locale.PARRY, locale.PARRY2);
	HT_SetStat(prefix, locale.LMISS, missP3, missP2, missP0, locale.MISS, locale.MISS2);
	HT_SetStat(prefix, locale.LBLOCK, blockP3, nil, nil, locale.BLOCK, locale.BLOCK2:format(GetShieldBlock()));
	HT_SetStat(prefix, locale.LHIT, hitP3, hitP2, hitP0, locale.HIT, locale.HIT2);
	while statLine <= 6 do
		HT_SetStat(prefix);
	end
end

SLASH_HITTABLE1, SLASH_HITTABLE2, SLASH_HITTABLE3 = "/hittable", "/ht", "/hits";
function SlashCmdList.HITTABLE(msg)
	local delta = tonumber(msg) or 3;
	local res = locale.SLASH:format(UnitLevel("player")+delta, getIncomingHitTable(delta));
	res = res:gsub(" 0[.,]00%% [^,.]+", ""):gsub(",+", ","):gsub(":[.,]", ":"):gsub(",%.", ".");
	print("|cff459EF7" .. res .. "|r");
end

hooksecurefunc("UpdatePaperdollStats", HT_Hook)
table.insert(PLAYERSTAT_DROPDOWN_OPTIONS, "HT_HITTABLE");